from uuid import uuid4
from collections import defaultdict
from pyproj import Proj


class visualdatalayer():

    def __init__(self, tile, uuid):
        self.tile = tile
        self.uuid = uuid
        self.proj = Proj(tile.cell.proj)

    def as_treelayer(self):

        return f'''<layer-tree-layer checked="Qt::Checked" 
            id="{self.tile.acq_id}_visual_tif_{self.uuid}" 
            expanded="1" name="{self.tile.acq_id}-visual.tif" 
            legend_exp="" providerKey="gdal" 
            source="{self.tile.gdal_prefix}{self.tile.asset_paths['visual']}">
        <customproperties/>
        </layer-tree-layer>'''

    def as_maplayer(self):
        return f'''
        <maplayer maxScale="0" autoRefreshTime="0" type="raster" hasScaleBasedVisibilityFlag="0" autoRefreshEnabled="0" styleCategories="AllStyleCategories" minScale="1e+08" refreshOnNotifyEnabled="0" refreshOnNotifyMessage="">
      <extent>
        <xmin>{self.tile.cell.bounds[0] - 0.30517578125 * 256} </xmin>
        <ymin>{self.tile.cell.bounds[1] - 0.30517578125 * 256}</ymin>
        <xmax>{self.tile.cell.bounds[2] + 0.30517578125 * 256}</xmax>
        <ymax>{self.tile.cell.bounds[3] + 0.30517578125 * 256 }</ymax>
      </extent>
      <id>{self.tile.acq_id}_visual_tif_{self.uuid}</id>
      <datasource>{self.tile.gdal_prefix}{self.tile.asset_paths['visual']}</datasource>
      <keywordList>
        <value></value>
      </keywordList>
      <layername>{self.tile.acq_id}-visual.tif</layername>
      <srs>
        <spatialrefsys>
          <wkt>{self.proj.crs.to_wkt()}</wkt>
          <proj4>{self.proj.srs}</proj4>
          <srsid>3094</srsid>
          <srid>{self.proj.crs.to_epsg()}</srid>
          <authid>{self.proj.crs.to_string()}</authid>
          <description>{self.proj.crs.name}</description>
          <projectionacronym>utm</projectionacronym>
          <ellipsoidacronym>WGS84</ellipsoidacronym>
          <geographicflag>false</geographicflag>
        </spatialrefsys>
      </srs>
      <resourceMetadata>
        <identifier></identifier>
        <parentidentifier></parentidentifier>
        <language></language>
        <type></type>
        <title></title>
        <abstract></abstract>
        <links/>
        <fees></fees>
        <encoding></encoding>
        <crs>
          <spatialrefsys>
            <wkt>{self.proj.crs.to_wkt()}</wkt>
            <proj4>{self.proj.srs}</proj4>
            <srsid>3094</srsid>
            <srid>{self.proj.crs.to_epsg()}</srid>
            <authid>{self.proj.crs.to_string()}</authid>
            <description>{self.proj.crs.name}</description>
            <projectionacronym>utm</projectionacronym>
            <ellipsoidacronym>WGS84</ellipsoidacronym>
            <geographicflag>false</geographicflag>
            </spatialrefsys>
        </crs>
        <extent/>
      </resourceMetadata>
      <provider>gdal</provider>
      <noData>
        <noDataList bandNo="1" useSrcNoData="0"/>
        <noDataList bandNo="2" useSrcNoData="0"/>
        <noDataList bandNo="3" useSrcNoData="0"/>
        <noDataList bandNo="4" useSrcNoData="0"/>
      </noData>
      <map-layer-style-manager current="default">
        <map-layer-style name="default"/>
      </map-layer-style-manager>
      <flags>
        <Identifiable>1</Identifiable>
        <Removable>1</Removable>
        <Searchable>1</Searchable>
      </flags>
      <customproperties>
        <property key="identify/format" value="Value"/>
      </customproperties>
      <pipe>
        <rasterrenderer type="multibandcolor" alphaBand="4" opacity="1" redBand="1" blueBand="3" nodataColor="" greenBand="2">
          <rasterTransparency/>
          <minMaxOrigin>
            <limits>MinMax</limits>
            <extent>WholeRaster</extent>
            <statAccuracy>Estimated</statAccuracy>
            <cumulativeCutLower>0.02</cumulativeCutLower>
            <cumulativeCutUpper>0.98</cumulativeCutUpper>
            <stdDevFactor>2</stdDevFactor>
          </minMaxOrigin>
          <redContrastEnhancement>
            <minValue>0</minValue>
            <maxValue>255</maxValue>
            <algorithm>NoEnhancement</algorithm>
          </redContrastEnhancement>
          <greenContrastEnhancement>
            <minValue>0</minValue>
            <maxValue>255</maxValue>
            <algorithm>NoEnhancement</algorithm>
          </greenContrastEnhancement>
          <blueContrastEnhancement>
            <minValue>0</minValue>
            <maxValue>255</maxValue>
            <algorithm>NoEnhancement</algorithm>
          </blueContrastEnhancement>
        </rasterrenderer>
        <brightnesscontrast brightness="0" contrast="0"/>
        <huesaturation colorizeOn="0" colorizeGreen="128" saturation="0" grayscaleMode="0" colorizeRed="255" colorizeStrength="100" colorizeBlue="128"/>
        <rasterresampler maxOversampling="2"/>
      </pipe>
      <blendMode>0</blendMode>
    </maplayer>'''


class clouddatalayer():

    def __init__(self, tile, uuid):
        self.tile = tile
        self.uuid = uuid
        self.proj = Proj(tile.cell.proj)

    def as_treelayer(self):

        return f'''<layer-tree-layer checked="Qt::Checked" 
            id="{self.tile.acq_id}_cloud_mask_{self.tile.cell.zone}_{self.tile.cell.quadkey}_{self.tile.acq_id}_clouds_mask_{self.uuid}" 
            expanded="1" name="{self.tile.acq_id}-cloud-mask {self.tile.cell.zone}/{self.tile.cell.quadkey}/self-clouds-mask" 
            legend_exp="" providerKey="ogr" 
            source="{self.tile.gdal_prefix}{self.tile.asset_paths['clouds']}|layername={self.tile.cell.zone}/{self.tile.cell.quadkey}/{self.tile.acq_id}-clouds-mask">
        <customproperties/>
        </layer-tree-layer>'''

    def as_maplayer(self):
        return f'''<maplayer refreshOnNotifyEnabled="0" simplifyAlgorithm="0" simplifyMaxScale="1" autoRefreshTime="0" minScale="100000000" wkbType="MultiPolygon" simplifyDrawingHints="1" readOnly="0" maxScale="0" refreshOnNotifyMessage="" styleCategories="AllStyleCategories" simplifyDrawingTol="1" labelsEnabled="0" type="vector" simplifyLocal="1" hasScaleBasedVisibilityFlag="0" autoRefreshEnabled="0" geometry="Polygon">
      <id>{self.tile.acq_id}_cloud_mask_{self.tile.cell.zone}_{self.tile.cell.quadkey}_{self.tile.acq_id}_clouds_mask_{self.uuid}</id>
      <datasource>{self.tile.gdal_prefix}{self.tile.asset_paths['clouds']}|layername={self.tile.cell.zone}/{self.tile.cell.quadkey}/{self.tile.acq_id}-cloud-mask</datasource>
      <keywordList>
        <value></value>
      </keywordList>
      <layername>{self.tile.acq_id}-cloud-mask 10/{self.tile.cell.quadkey}/{self.tile.acq_id}-cloud-mask</layername>
      <srs>
        <spatialrefsys>
          <wkt>{self.proj.crs.to_wkt()}</wkt>
          <proj4>{self.proj.srs}</proj4>
          <srsid>3094</srsid>
          <srid>{self.proj.crs.to_epsg()}</srid>
          <authid>{self.proj.crs.to_string()}</authid>
          <description>{self.proj.crs.name}</description>
          <projectionacronym>utm</projectionacronym>
          <ellipsoidacronym>WGS84</ellipsoidacronym>
          <geographicflag>false</geographicflag> 
        </spatialrefsys>
      </srs>
      <provider encoding="UTF-8">ogr</provider>
      <legend type="default-vector"/>
      <map-layer-style-manager current="default">
        <map-layer-style name="default"/>
      </map-layer-style-manager>
      <flags>
        <Identifiable>1</Identifiable>
        <Removable>1</Removable>
        <Searchable>1</Searchable>
      </flags>
      <renderer-v2 type="singleSymbol" enableorderby="0" forceraster="0" symbollevels="0">
        <symbols>
          <symbol type="fill" alpha="1" clip_to_extent="1" force_rhr="0" name="0">
            <layer locked="0" enabled="1" pass="0" class="SimpleFill">
              <prop k="border_width_map_unit_scale" v="3x:0,0,0,0,0,0"/>
              <prop k="color" v="229,182,54,255"/>
              <prop k="joinstyle" v="bevel"/>
              <prop k="offset" v="0,0"/>
              <prop k="offset_map_unit_scale" v="3x:0,0,0,0,0,0"/>
              <prop k="offset_unit" v="MM"/>
              <prop k="outline_color" v="35,35,35,255"/>
              <prop k="outline_style" v="solid"/>
              <prop k="outline_width" v="0.26"/>
              <prop k="outline_width_unit" v="MM"/>
              <prop k="style" v="solid"/>
              <data_defined_properties>
                <Option type="Map">
                  <Option type="QString" value="" name="name"/>
                  <Option name="properties"/>
                  <Option type="QString" value="collection" name="type"/>
                </Option>
              </data_defined_properties>
            </layer>
          </symbol>
        </symbols>
        <rotation/>
        <sizescale/>
      </renderer-v2>
      <blendMode>0</blendMode>
      <featureBlendMode>0</featureBlendMode>
      <layerOpacity>1</layerOpacity>
      <geometryOptions geometryPrecision="0" removeDuplicateNodes="0">
        <activeChecks type="StringList">
          <Option type="QString" value=""/>
        </activeChecks>
        <checkConfiguration/>
      </geometryOptions>
      <fieldConfiguration>
        <field name="fid">
          <editWidget type="">
            <config>
              <Option/>
            </config>
          </editWidget>
        </field>
        <field name="cloudCoverPct">
          <editWidget type="">
            <config>
              <Option/>
            </config>
          </editWidget>
        </field>
      </fieldConfiguration>
      <aliases>
        <alias field="fid" index="0" name=""/>
        <alias field="cloudCoverPct" index="1" name=""/>
      </aliases>
      <defaults>
        <default field="fid" expression="" applyOnUpdate="0"/>
        <default field="cloudCoverPct" expression="" applyOnUpdate="0"/>
      </defaults>
      <constraints>
        <constraint field="fid" notnull_strength="1" unique_strength="1" exp_strength="0" constraints="3"/>
        <constraint field="cloudCoverPct" notnull_strength="0" unique_strength="0" exp_strength="0" constraints="0"/>
      </constraints>
      <constraintExpressions>
        <constraint field="fid" desc="" exp=""/>
        <constraint field="cloudCoverPct" desc="" exp=""/>
      </constraintExpressions>
      <expressionfields/>
      <attributeactions>
        <defaultAction key="Canvas" value="{00000000-0000-0000-0000-000000000000}"/>
      </attributeactions>
      <attributetableconfig actionWidgetStyle="dropDown" sortOrder="0" sortExpression="">
        <columns/>
      </attributetableconfig>
      <conditionalstyles>
        <rowstyles/>
        <fieldstyles/>
      </conditionalstyles>
      <editform tolerant="1"></editform>
      <editforminit/>
      <editforminitcodesource>0</editforminitcodesource>
      <editforminitfilepath></editforminitfilepath>
      <editforminitcode><![CDATA[]]></editforminitcode>
      <featformsuppress>0</featformsuppress>
      <editorlayout>generatedlayout</editorlayout>
      <editable/>
      <labelOnTop/>
      <previewExpression></previewExpression>
      <mapTip></mapTip>
    </maplayer> '''

class treelayer():

    def __init__(self, tile, uuid):
        self.tile = tile
        self.uuid = uuid
        self.visual = visualdatalayer(tile, uuid)
        self.clouds = clouddatalayer(tile, uuid)

    def as_text(self):
        return f'''
            <layer-tree-group checked="Qt::Checked" expanded="1" name="{self.tile.cell.id}">
                <customproperties/>
                {self.clouds.as_treelayer()}
                {self.visual.as_treelayer()}
            </layer-tree-group> '''

class maplayer():

    def __init__(self, tile, uuid):
        self.tile = tile
        self.uuid = uuid
        self.visual = visualdatalayer(tile, uuid)
        self.clouds = clouddatalayer(tile, uuid)
        
    def as_text(self):
        return f'''
                {self.clouds.as_maplayer()}
                {self.visual.as_maplayer()}
             '''


class qlayer():

    def __init__(self, collection):
        self.treelayers = defaultdict(list) 
        self.maplayers = []
        self.zone_ids = [(z, uuid4().hex) for z in collection.zones]
        for acq in collection.acquisitions:
            for tile in acq.tiles:
                uuid = uuid4().hex
                self.treelayers[acq].append(treelayer(tile, uuid))
                self.maplayers.append(maplayer(tile, uuid))


    def render_tree(self):
        text = ''
        for acq, layers in self.treelayers.items():
            text += f'<layer-tree-group checked="Qt::Unchecked" expanded="0" name="{acq.acq_id}">'
            text += '<customproperties/>'
            text += '\n'.join(t.as_text() for t in layers)
            text += ' </layer-tree-group>'
        return text

    def render_layers(self):
        return '\n'.join([l.as_text() for l in self.maplayers])

    def render_zones_tree(self):
        text = f'<layer-tree-group checked="Qt::Unchecked" expanded="0" name="ARD Grids">'
        text += '<customproperties/>'
        for zone, uuid in self.zone_ids:
            text += f'''
            <layer-tree-layer legend_exp="" source="/vsis3/ard-docs/main/canvas-grids/Z{zone}.gpkg|layername=Z{zone}" providerKey="ogr" expanded="1" name="ARD Grid Zone {zone}" checked="Qt::Unchecked" id="Z{zone}_{uuid}">
            <customproperties/>
            </layer-tree-layer>
            '''
        text += ' </layer-tree-group>'
        return text

    def render_zones_layers(self):
        text = ''
        for zone, uuid in self.zone_ids:
            proj = Proj(f'EPSG:326{zone:02}')
            text += f'''
            <maplayer simplifyDrawingHints="1" refreshOnNotifyMessage="" simplifyLocal="1" autoRefreshTime="0" simplifyDrawingTol="1" readOnly="0" type="vector" simplifyAlgorithm="0" styleCategories="AllStyleCategories" maxScale="0" minScale="100000000" simplifyMaxScale="1" labelsEnabled="1" wkbType="Polygon" geometry="Polygon" refreshOnNotifyEnabled="0" hasScaleBasedVisibilityFlag="0" autoRefreshEnabled="0">
      <extent>
        <xmin>165000</xmin>
        <ymin>-9330000</ymin>
        <xmax>835000</xmax>
        <ymax>9330000</ymax>
      </extent>
      <id>Z{zone}_{uuid}</id>
      <datasource>/vsis3/ard-docs/main/canvas-grids/Z{zone}.gpkg|layername=Z{zone}</datasource>
      <keywordList>
        <value></value>
      </keywordList>
      <layername>ARD Grid Zone {zone}</layername>
      <srs>
       <spatialrefsys>
          <wkt>{proj.crs.to_wkt()}</wkt>
          <proj4>{proj.srs}</proj4>
          <srsid>3094</srsid>
          <srid>{proj.crs.to_epsg()}</srid>
          <authid>{proj.crs.to_string()}</authid>
          <description>{proj.crs.name}</description>
          <projectionacronym>utm</projectionacronym>
          <ellipsoidacronym>WGS84</ellipsoidacronym>
          <geographicflag>false</geographicflag> 
        </spatialrefsys> 
      </srs>
      <provider encoding="UTF-8">ogr</provider>
      <legend type="default-vector"/>
      <map-layer-style-manager current="default">
        <map-layer-style name="default"/>
      </map-layer-style-manager>
      <flags>
        <Identifiable>1</Identifiable>
        <Removable>1</Removable>
        <Searchable>1</Searchable>
      </flags>
      <renderer-v2 type="singleSymbol" forceraster="0" symbollevels="0" enableorderby="0">
        <symbols>
          <symbol type="fill" force_rhr="0" alpha="1" name="0" clip_to_extent="1">
            <layer pass="0" class="SimpleFill" enabled="1" locked="0">
              <prop k="border_width_map_unit_scale" v="3x:0,0,0,0,0,0"/>
              <prop k="color" v="0,0,255,255"/>
              <prop k="joinstyle" v="bevel"/>
              <prop k="offset" v="0,0"/>
              <prop k="offset_map_unit_scale" v="3x:0,0,0,0,0,0"/>
              <prop k="offset_unit" v="MM"/>
              <prop k="outline_color" v="227,69,72,255"/>
              <prop k="outline_style" v="solid"/>
              <prop k="outline_width" v="0.66"/>
              <prop k="outline_width_unit" v="MM"/>
              <prop k="style" v="no"/>
              <data_defined_properties>
                <Option type="Map">
                  <Option type="QString" value="" name="name"/>
                  <Option name="properties"/>
                  <Option type="QString" value="collection" name="type"/>
                </Option>
              </data_defined_properties>
            </layer>
          </symbol>
        </symbols>
        <rotation/>
        <sizescale/>
      </renderer-v2>
      <labeling type="simple">
        <settings calloutType="simple">
          <text-style fontSizeMapUnitScale="3x:0,0,0,0,0,0" fontLetterSpacing="0" useSubstitutions="0" textOrientation="horizontal" fontCapitals="0" fontFamily=".SF NS Text" fontKerning="1" blendMode="0" fontStrikeout="0" previewBkgrdColor="255,255,255,255" fontSize="10" fontItalic="0" multilineHeight="1" fieldName="utmzone + '-' + quadkey" fontWeight="50" fontUnderline="0" textOpacity="1" namedStyle="Regular" isExpression="1" fontSizeUnit="Point" textColor="0,0,0,255" fontWordSpacing="0">
            <text-buffer bufferDraw="1" bufferNoFill="0" bufferJoinStyle="128" bufferSizeMapUnitScale="3x:0,0,0,0,0,0" bufferBlendMode="0" bufferOpacity="1" bufferColor="255,255,255,255" bufferSize="1" bufferSizeUnits="MM"/>
            <text-mask maskOpacity="1" maskEnabled="0" maskJoinStyle="128" maskSizeMapUnitScale="3x:0,0,0,0,0,0" maskSize="1.5" maskType="0" maskSizeUnits="MM" maskedSymbolLayers=""/>
            <background shapeDraw="0" shapeSizeX="0" shapeOpacity="1" shapeOffsetUnit="MM" shapeOffsetMapUnitScale="3x:0,0,0,0,0,0" shapeOffsetY="0" shapeOffsetX="0" shapeSizeMapUnitScale="3x:0,0,0,0,0,0" shapeRadiiX="0" shapeRadiiUnit="MM" shapeFillColor="255,255,255,255" shapeBorderWidth="0" shapeJoinStyle="64" shapeBlendMode="0" shapeBorderWidthMapUnitScale="3x:0,0,0,0,0,0" shapeSizeType="0" shapeRadiiMapUnitScale="3x:0,0,0,0,0,0" shapeBorderColor="128,128,128,255" shapeSVGFile="" shapeSizeUnit="MM" shapeRotation="0" shapeType="0" shapeSizeY="0" shapeRotationType="0" shapeBorderWidthUnit="MM" shapeRadiiY="0">
              <symbol type="marker" force_rhr="0" alpha="1" name="markerSymbol" clip_to_extent="1">
                <layer pass="0" class="SimpleMarker" enabled="1" locked="0">
                  <prop k="angle" v="0"/>
                  <prop k="color" v="196,60,57,255"/>
                  <prop k="horizontal_anchor_point" v="1"/>
                  <prop k="joinstyle" v="bevel"/>
                  <prop k="name" v="circle"/>
                  <prop k="offset" v="0,0"/>
                  <prop k="offset_map_unit_scale" v="3x:0,0,0,0,0,0"/>
                  <prop k="offset_unit" v="MM"/>
                  <prop k="outline_color" v="35,35,35,255"/>
                  <prop k="outline_style" v="solid"/>
                  <prop k="outline_width" v="0"/>
                  <prop k="outline_width_map_unit_scale" v="3x:0,0,0,0,0,0"/>
                  <prop k="outline_width_unit" v="MM"/>
                  <prop k="scale_method" v="diameter"/>
                  <prop k="size" v="2"/>
                  <prop k="size_map_unit_scale" v="3x:0,0,0,0,0,0"/>
                  <prop k="size_unit" v="MM"/>
                  <prop k="vertical_anchor_point" v="1"/>
                  <data_defined_properties>
                    <Option type="Map">
                      <Option type="QString" value="" name="name"/>
                      <Option name="properties"/>
                      <Option type="QString" value="collection" name="type"/>
                    </Option>
                  </data_defined_properties>
                </layer>
              </symbol>
            </background>
            <shadow shadowRadius="1.5" shadowUnder="0" shadowRadiusMapUnitScale="3x:0,0,0,0,0,0" shadowColor="0,0,0,255" shadowDraw="0" shadowOffsetAngle="135" shadowOffsetDist="1" shadowScale="100" shadowBlendMode="6" shadowRadiusAlphaOnly="0" shadowOffsetGlobal="1" shadowRadiusUnit="MM" shadowOpacity="0.7" shadowOffsetUnit="MM" shadowOffsetMapUnitScale="3x:0,0,0,0,0,0"/>
            <dd_properties>
              <Option type="Map">
                <Option type="QString" value="" name="name"/>
                <Option name="properties"/>
                <Option type="QString" value="collection" name="type"/>
              </Option>
            </dd_properties>
            <substitutions/>
          </text-style>
          <text-format multilineAlign="0" addDirectionSymbol="0" reverseDirectionSymbol="0" autoWrapLength="0" formatNumbers="0" plussign="0" wrapChar="" rightDirectionSymbol=">" decimals="3" leftDirectionSymbol="&lt;" placeDirectionSymbol="0" useMaxLineLengthForAutoWrap="1"/>
          <placement fitInPolygonOnly="0" distMapUnitScale="3x:0,0,0,0,0,0" rotationAngle="0" labelOffsetMapUnitScale="3x:0,0,0,0,0,0" maxCurvedCharAngleIn="25" predefinedPositionOrder="TR,TL,BR,BL,R,L,TSR,BSR" distUnits="MM" placement="0" placementFlags="10" geometryGeneratorEnabled="0" preserveRotation="1" geometryGenerator="" quadOffset="0" offsetUnits="MM" priority="5" geometryGeneratorType="PointGeometry" offsetType="0" centroidWhole="1" repeatDistance="0" dist="0" xOffset="0" yOffset="0" centroidInside="0" repeatDistanceUnits="MM" repeatDistanceMapUnitScale="3x:0,0,0,0,0,0" maxCurvedCharAngleOut="-25" layerType="PolygonGeometry" overrunDistance="0" overrunDistanceUnit="MM" overrunDistanceMapUnitScale="3x:0,0,0,0,0,0"/>
          <rendering fontMinPixelSize="3" minFeatureSize="0" drawLabels="1" displayAll="0" scaleVisibility="1" maxNumLabels="2000" mergeLines="0" obstacleFactor="1" obstacleType="1" labelPerPart="0" scaleMin="0" upsidedownLabels="0" obstacle="1" zIndex="0" scaleMax="134345" fontMaxPixelSize="10000" fontLimitPixelSize="0" limitNumLabels="0"/>
          <dd_properties>
            <Option type="Map">
              <Option type="QString" value="" name="name"/>
              <Option name="properties"/>
              <Option type="QString" value="collection" name="type"/>
            </Option>
          </dd_properties>
          <callout type="simple">
            <Option type="Map">
              <Option type="QString" value="pole_of_inaccessibility" name="anchorPoint"/>
              <Option type="Map" name="ddProperties">
                <Option type="QString" value="" name="name"/>
                <Option name="properties"/>
                <Option type="QString" value="collection" name="type"/>
              </Option>
              <Option type="bool" value="false" name="drawToAllParts"/>
              <Option type="QString" value="0" name="enabled"/>
              <Option type="QString" value="&lt;symbol type=&quot;line&quot; force_rhr=&quot;0&quot; alpha=&quot;1&quot; name=&quot;symbol&quot; clip_to_extent=&quot;1&quot;>&lt;layer pass=&quot;0&quot; class=&quot;SimpleLine&quot; enabled=&quot;1&quot; locked=&quot;0&quot;>&lt;prop k=&quot;capstyle&quot; v=&quot;square&quot;/>&lt;prop k=&quot;customdash&quot; v=&quot;5;2&quot;/>&lt;prop k=&quot;customdash_map_unit_scale&quot; v=&quot;3x:0,0,0,0,0,0&quot;/>&lt;prop k=&quot;customdash_unit&quot; v=&quot;MM&quot;/>&lt;prop k=&quot;draw_inside_polygon&quot; v=&quot;0&quot;/>&lt;prop k=&quot;joinstyle&quot; v=&quot;bevel&quot;/>&lt;prop k=&quot;line_color&quot; v=&quot;60,60,60,255&quot;/>&lt;prop k=&quot;line_style&quot; v=&quot;solid&quot;/>&lt;prop k=&quot;line_width&quot; v=&quot;0.3&quot;/>&lt;prop k=&quot;line_width_unit&quot; v=&quot;MM&quot;/>&lt;prop k=&quot;offset&quot; v=&quot;0&quot;/>&lt;prop k=&quot;offset_map_unit_scale&quot; v=&quot;3x:0,0,0,0,0,0&quot;/>&lt;prop k=&quot;offset_unit&quot; v=&quot;MM&quot;/>&lt;prop k=&quot;ring_filter&quot; v=&quot;0&quot;/>&lt;prop k=&quot;use_custom_dash&quot; v=&quot;0&quot;/>&lt;prop k=&quot;width_map_unit_scale&quot; v=&quot;3x:0,0,0,0,0,0&quot;/>&lt;data_defined_properties>&lt;Option type=&quot;Map&quot;>&lt;Option type=&quot;QString&quot; value=&quot;&quot; name=&quot;name&quot;/>&lt;Option name=&quot;properties&quot;/>&lt;Option type=&quot;QString&quot; value=&quot;collection&quot; name=&quot;type&quot;/>&lt;/Option>&lt;/data_defined_properties>&lt;/layer>&lt;/symbol>" name="lineSymbol"/>
              <Option type="double" value="0" name="minLength"/>
              <Option type="QString" value="3x:0,0,0,0,0,0" name="minLengthMapUnitScale"/>
              <Option type="QString" value="MM" name="minLengthUnit"/>
              <Option type="double" value="0" name="offsetFromAnchor"/>
              <Option type="QString" value="3x:0,0,0,0,0,0" name="offsetFromAnchorMapUnitScale"/>
              <Option type="QString" value="MM" name="offsetFromAnchorUnit"/>
              <Option type="double" value="0" name="offsetFromLabel"/>
              <Option type="QString" value="3x:0,0,0,0,0,0" name="offsetFromLabelMapUnitScale"/>
              <Option type="QString" value="MM" name="offsetFromLabelUnit"/>
            </Option>
          </callout>
        </settings>
      </labeling>
      <customproperties>
        <property value="0" key="embeddedWidgets/count"/>
        <property key="variableNames"/>
        <property key="variableValues"/>
      </customproperties>
      <blendMode>0</blendMode>
      <featureBlendMode>0</featureBlendMode>
      <layerOpacity>1</layerOpacity>
      <SingleCategoryDiagramRenderer diagramType="Histogram" attributeLegend="1">
        <DiagramCategory sizeType="MM" backgroundAlpha="255" penColor="#000000" sizeScale="3x:0,0,0,0,0,0" penWidth="0" spacingUnit="MM" showAxis="1" width="15" rotationOffset="270" spacingUnitScale="3x:0,0,0,0,0,0" enabled="0" opacity="1" diagramOrientation="Up" barWidth="5" maxScaleDenominator="1e+08" labelPlacementMethod="XHeight" lineSizeType="MM" minimumSize="0" spacing="5" scaleBasedVisibility="0" direction="0" minScaleDenominator="0" penAlpha="255" lineSizeScale="3x:0,0,0,0,0,0" scaleDependency="Area" backgroundColor="#ffffff" height="15">
          <fontProperties description=".SF NS Text,13,-1,5,50,0,0,0,0,0" style=""/>
          <axisSymbol>
            <symbol type="line" force_rhr="0" alpha="1" name="" clip_to_extent="1">
              <layer pass="0" class="SimpleLine" enabled="1" locked="0">
                <prop k="capstyle" v="square"/>
                <prop k="customdash" v="5;2"/>
                <prop k="customdash_map_unit_scale" v="3x:0,0,0,0,0,0"/>
                <prop k="customdash_unit" v="MM"/>
                <prop k="draw_inside_polygon" v="0"/>
                <prop k="joinstyle" v="bevel"/>
                <prop k="line_color" v="35,35,35,255"/>
                <prop k="line_style" v="solid"/>
                <prop k="line_width" v="0.26"/>
                <prop k="line_width_unit" v="MM"/>
                <prop k="offset" v="0"/>
                <prop k="offset_map_unit_scale" v="3x:0,0,0,0,0,0"/>
                <prop k="offset_unit" v="MM"/>
                <prop k="ring_filter" v="0"/>
                <prop k="use_custom_dash" v="0"/>
                <prop k="width_map_unit_scale" v="3x:0,0,0,0,0,0"/>
                <data_defined_properties>
                  <Option type="Map">
                    <Option type="QString" value="" name="name"/>
                    <Option name="properties"/>
                    <Option type="QString" value="collection" name="type"/>
                  </Option>
                </data_defined_properties>
              </layer>
            </symbol>
          </axisSymbol>
        </DiagramCategory>
      </SingleCategoryDiagramRenderer>
      <DiagramLayerSettings linePlacementFlags="18" zIndex="0" dist="0" placement="1" priority="0" showAll="1" obstacle="0">
        <properties>
          <Option type="Map">
            <Option type="QString" value="" name="name"/>
            <Option name="properties"/>
            <Option type="QString" value="collection" name="type"/>
          </Option>
        </properties>
      </DiagramLayerSettings>
      <geometryOptions geometryPrecision="0" removeDuplicateNodes="0">
        <activeChecks/>
        <checkConfiguration type="Map">
          <Option type="Map" name="QgsGeometryGapCheck">
            <Option type="double" value="0" name="allowedGapsBuffer"/>
            <Option type="bool" value="false" name="allowedGapsEnabled"/>
            <Option type="QString" value="" name="allowedGapsLayer"/>
          </Option>
        </checkConfiguration>
      </geometryOptions>
      <fieldConfiguration>
        <field name="fid">
          <editWidget type="TextEdit">
            <config>
              <Option/>
            </config>
          </editWidget>
        </field>
        <field name="utmzone">
          <editWidget type="TextEdit">
            <config>
              <Option/>
            </config>
          </editWidget>
        </field>
        <field name="quadkey">
          <editWidget type="TextEdit">
            <config>
              <Option/>
            </config>
          </editWidget>
        </field>
      </fieldConfiguration>
      <aliases>
        <alias index="0" name="" field="fid"/>
        <alias index="1" name="" field="utmzone"/>
        <alias index="2" name="" field="quadkey"/>
      </aliases>
      <defaults>
        <default expression="" field="fid" applyOnUpdate="0"/>
        <default expression="" field="utmzone" applyOnUpdate="0"/>
        <default expression="" field="quadkey" applyOnUpdate="0"/>
      </defaults>
      <constraints>
        <constraint exp_strength="0" constraints="3" notnull_strength="1" unique_strength="1" field="fid"/>
        <constraint exp_strength="0" constraints="0" notnull_strength="0" unique_strength="0" field="utmzone"/>
        <constraint exp_strength="0" constraints="0" notnull_strength="0" unique_strength="0" field="quadkey"/>
      </constraints>
      <constraintExpressions>
        <constraint exp="" desc="" field="fid"/>
        <constraint exp="" desc="" field="utmzone"/>
        <constraint exp="" desc="" field="quadkey"/>
      </constraintExpressions>
      <expressionfields/>
      <attributeactions>
        <defaultAction value="{00000000-0000-0000-0000-000000000000}" key="Canvas"/>
      </attributeactions>
      <attributetableconfig sortOrder="0" sortExpression="" actionWidgetStyle="dropDown">
        <columns>
          <column type="field" hidden="0" name="fid" width="-1"/>
          <column type="field" hidden="0" name="utmzone" width="-1"/>
          <column type="field" hidden="0" name="quadkey" width="-1"/>
          <column type="actions" hidden="1" width="-1"/>
        </columns>
      </attributetableconfig>
      <editform tolerant="1"></editform>
      <editforminitcodesource>0</editforminitcodesource>
      <featformsuppress>0</featformsuppress>
      <editorlayout>generatedlayout</editorlayout>
      <editable>
        <field editable="1" name="fid"/>
        <field editable="1" name="quadkey"/>
        <field editable="1" name="utmzone"/>
      </editable>
      <labelOnTop>
        <field name="fid" labelOnTop="0"/>
        <field name="quadkey" labelOnTop="0"/>
        <field name="utmzone" labelOnTop="0"/>
      </labelOnTop>
      <previewExpression>fid</previewExpression>
      <mapTip></mapTip>
    </maplayer>'''

        return text

    def as_text(self):
        return f'''<!DOCTYPE qgis-layer-definition>
        <qlr>
           <layer-tree-group checked="Qt::Checked" expanded="1" name="">
            <customproperties/>
            {self.render_zones_tree()}
            {self.render_tree()}
            </layer-tree-group> 
            <maplayers>
            {self.render_zones_layers()}
            {self.render_layers()}
            </maplayers>
        </qlr> ''' 